/* Keep these headers */
#include <stdbool.h>
#include <stddef.h>
#include <stdint.h>
#include <tice.h>

/* Standard headers - it's recommended to leave them included */
#include <math.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

/* Shared libraries */
#include <lib/ce/graphx.h>
#include <lib/ce/keypadc.h>
#include "gfx/cookclik_gfx.h"

/* Timer defines */
#define ONE_SECOND         32768/1
#define HALF_SECOND        32768/2
#define QUARTER_SECOND     32768/4

/* Palette defines */
#define TRANSPARENT	0
#define BLACK	58
#define BLUE_1	60
#define BUTTON_GRAY	74
#define WOOD_BROWN	171
#define WHITE	255

/* Function defines */
#define within_box(x1,y1,x2,y2)	(cursor_x >= (x1) && cursor_x <= (x2) && cursor_y >= (y1) && cursor_y <= (y2))

/* Other defines */
#define CURSOR	0
#define GRANDMA	1
#define FARM	2
#define FACTORY	3
#define MINE	4
#define BANK	5
#define TEMPLE	6
#define WIZARD_TOWER	7
#define SHIPMENT	8
#define ALCHEMY_LAB	9
#define PORTAL	10
#define TIME_MACHINE	11
#define ANTIMATTER_CONDENSER	12
#define PRISM	13

/* Global variable declarations and whatnot; should be self-explanatory */

const gfx_image_t *building_icons[] = {
	building_cursor,
	building_grandma,
	building_farm,
	building_factory,
	building_mine,
	building_bank,
	building_temple,
	building_wizard_tower,
	building_shipment,
	building_alchemy_lab,
	building_portal,
	building_time_machine,
	building_antimatter_condenser,
	building_prism,
};

const char *building_names[] = {
	"Cursor",
	"Grandma",
	"Farm",
	"Factory",
	"Mine",
	"Bank",
	"Temple",
	"Wizard Tower",
	"Shipment",
	"Alchemy Lab",
	"Portal",
	"Time Machine",
	"Antimatter Condenser",
	"Prism",
};

gfx_image_t *cookies_back;
gfx_image_t *cursor_back;

uint8_t temp[3], buildings_scroll;
bool click_debounce;

uint16_t cursor_x, cursor_y;

double cookies, cps;

bool buying;

/* Function prototypes */
void click(void);
void draw_all_graphics(void);
void draw_buildings(void);
void update_graphics(void);
bool tick_timer(void);

/* Main function (actual code) */
void main(void) {
	kb_key_t key7;
	
	/* Allocate space for the temporary sprite to save the background */
	malloc(0);
	cookies_back = gfx_MallocSprite( 88, 7 );
	cursor_back = gfx_MallocSprite( cursor->width, cursor->height );
	
	/* Initialize graphics */
	gfx_Begin(gfx_8bpp);
	
	/* Set up the palette */
	gfx_SetPalette(cookclik_gfx_pal, sizeof(cookclik_gfx_pal), 0);
	gfx_SetDrawBuffer();
	
	/* Disable everything in the timers so they don't run when we don't want them to */
	timer_Control = TIMER1_DISABLE;
	
	/* By using the 32768 kHz clock, we can count for exactly 1 second here, or a different interval of time */
	timer_1_ReloadValue = timer_1_Counter = ONE_SECOND;
	
	/* ------------------------ */
	/* Program code starts here */
	/* ------------------------ */
	
	draw_all_graphics();
	
	click_debounce = false;
	buildings_scroll = 0;
	
	/* --- */
	
	cursor_x = 0;
	cursor_y = 0;
	
	cookies = 0;
	cps = 1;
	
	buying = true;
	
	/* Enable the timer, set it to the 32768 kHz clock, enable an interrupt once it reaches 0, and make it count down */
    timer_Control = TIMER1_ENABLE | TIMER1_32K | TIMER1_0INT | TIMER1_DOWN;
	timer_IntStatus = TIMER1_RELOADED;
	
	do {
		kb_Scan();
		
		key7 = kb_Data[kb_group_7];
		
		/* Handle cursor movement */
		if ( key7 & kb_Left ) {
			if ( cursor_x ) { cursor_x--; }
		}
		if ( key7 & kb_Right ) {
			if ( cursor_x < 316 ) { cursor_x++; }
		}
		if ( key7 & kb_Up ) {
			if ( cursor_y ) { cursor_y--; }
		}
		if ( key7 & kb_Down ) {
			if ( cursor_y < 236 ) { cursor_y++; }
		}
		
		/* Register a cursor click */
		if ( kb_Data[kb_group_1] & kb_2nd || kb_Data[kb_group_6] & kb_Enter ) {
			if ( !click_debounce ) {
				click();
				click_debounce = true;
			}
		} else {
			click_debounce = false;
		}
		
		/* My current cookies per second scheme */
		/* I'll think of a better one I promise */
		if ( tick_timer() ) {
			cookies += cps;
		}
		
		update_graphics();
		
	} while ( !(kb_Data[kb_group_6] & kb_Clear ) );
	
	/* Clean up and exit */
	gfx_End(); prgm_CleanUp();
}


/* Functions */


/* Figure out what to do when you click, then do it */
void click(void) {
	/* Handle big cookie clicks */
	if ( within_box(0,95,89,184) ) {
		if ( perfect_cookie_data[ (cursor_y - 95) * perfect_cookie_data[0] + cursor_x + 2 ] ) {
			/* You have clicked on the big cookie */
			cookies++;
		}
	}
	
	/* Handle choosing between buying/selling */
	if ( within_box(192,37,230,53) ) {
		buying = cursor_y <= 45;
	}
	
	/* Handle scrolling through buildings */
	if ( within_box(191,54,319,59) || within_box(191,234,319,239) ) {
		if ( cursor_y <= 59 ) {
			if ( buildings_scroll ) {
				buildings_scroll--;
			}
		} else {
			if ( buildings_scroll < 7 ) {
				buildings_scroll++;
			}
		}
		draw_buildings();
	}
}

/* Draws everything on-screen */
void draw_all_graphics(void) {
	/* Draw the background */
	gfx_FillScreen( BLUE_1 );
	
	/* Text settings */
	gfx_SetTextFGColor( WHITE );
	gfx_SetTextBGColor( TRANSPARENT );
	gfx_SetTextTransparentColor( TRANSPARENT );
	
	/* Draw the "wood panelling" */
	gfx_SetColor( BLACK );
	gfx_Rectangle_NoClip( 0, 48, 190, 3 );
	gfx_Rectangle_NoClip( 90, 50, 3, 190 );
	gfx_Rectangle_NoClip( 92, 237, 98, 3 );
	gfx_Rectangle_NoClip( 189, 0, 3, 240 );
	gfx_Rectangle_NoClip( 191, 7, 129, 3 );
	gfx_Rectangle_NoClip( 191, 34, 129, 3 );
	gfx_SetColor( WOOD_BROWN );
	gfx_HorizLine_NoClip( 1, 49, 188 );
	gfx_VertLine_NoClip( 91, 51, 188 );
	gfx_HorizLine_NoClip( 93, 238, 96 );
	gfx_VertLine_NoClip( 190, 1, 238 );
	gfx_HorizLine_NoClip( 192, 8, 127 );
	gfx_HorizLine_NoClip( 192, 35, 127 );
	for ( temp[0] = 0; temp[0] < 6 ; temp[0]++ ) {
		gfx_SetColor( BLACK );
		gfx_Rectangle_NoClip( 92, temp[0] * 35 + 55, 98, 3 );
		gfx_SetColor( WOOD_BROWN );
		gfx_HorizLine_NoClip( 93, temp[0] * 35 + 56, 96 );
	}
	
	/* Draw the "buttons" */
	gfx_SetColor( BLACK );
	gfx_FillRectangle_NoClip( 191, 29, 129, 6 );
	gfx_FillRectangle_NoClip( 191, 54, 129, 6 );
	gfx_FillRectangle_NoClip( 191, 234, 129, 6 );
	gfx_SetColor( BUTTON_GRAY );
	gfx_FillRectangle_NoClip( 93, 51, 96, 4 );
	gfx_Sprite_NoClip( more_buildings, 113, 51 );
	gfx_Sprite_NoClip( arrow_up, 95, 51 );
	gfx_Sprite_NoClip( arrow_up, 183, 51 );
	gfx_FillRectangle_NoClip( 93, 233, 96, 4 );
	gfx_Sprite_NoClip( more_buildings, 113, 233 );
	gfx_Sprite_NoClip( arrow_down, 95, 233 );
	gfx_Sprite_NoClip( arrow_down, 183, 233 );
	gfx_FillRectangle_NoClip( 192, 30, 127, 4 );
	gfx_Sprite_NoClip( more_upgrades, 229, 30 );
	gfx_Sprite_NoClip( arrow_down, 193, 30 );
	gfx_Sprite_NoClip( arrow_down, 313, 30 );
	gfx_FillRectangle_NoClip( 192, 55, 127, 4 );
	gfx_Sprite_NoClip( more_buildings, 227, 55 );
	gfx_Sprite_NoClip( arrow_up, 193, 55 );
	gfx_Sprite_NoClip( arrow_up, 313, 55 );
	gfx_FillRectangle_NoClip( 192, 235, 127, 4 );
	gfx_Sprite_NoClip( more_buildings, 227, 235 );
	gfx_Sprite_NoClip( arrow_down, 193, 235 );
	gfx_Sprite_NoClip( arrow_down, 313, 235 );
	
	/* Draw the "buying/selling" buttons */
	gfx_FillRectangle_NoClip( 192, 37, 127, 17 );
	gfx_Sprite_NoClip( numbers_to_buy, 238, 41 );
	
	/* Draw the "frames" around the buildings */
	for ( temp[0] = 0; temp[0] < 7; temp[0]++ ) {
		gfx_SetColor( BLACK );
		gfx_Rectangle_NoClip( 191, temp[0] * 25 + 59, 129, 26 );
	}
	
	/* Self-explanatory */
	draw_buildings();
	
	/* Catch-all; apparently I need this */
	gfx_SetColor( BLACK );
	gfx_VertLine_NoClip( 319, 0, 54 );
	
	/* Draw the actual text */
	/* There will be more text */
	gfx_PrintStringXY( "Store", 236, 0 );
	gfx_PrintStringXY( "BUY", 193, 38 );
	gfx_PrintStringXY( "SELL", 193, 46 );
	
	/* Draw the giant cookie */
	/* The name "perfect" might make sense later */
	gfx_TransparentSprite_NoClip( perfect_cookie, 0, 95 );
}

/* Draw each of the building choices and their information */
void draw_buildings(void) {
	/* Text settings */
	gfx_SetTextFGColor( WHITE );
	gfx_SetTextBGColor( TRANSPARENT );
	gfx_SetTextTransparentColor( TRANSPARENT );
	
	gfx_SetColor( BUTTON_GRAY );
	for ( temp[0] = 0; temp[0] < 7; temp[0]++ ) {
		/* Effectively erase what was there before */
		gfx_FillRectangle_NoClip( 192, temp[0] * 25 + 60, 127, 24 );
		/* Draw building icon */
		gfx_TransparentSprite_NoClip( building_icons[temp[0] + buildings_scroll], 192, temp[0] * 25 + 60 );
		/* Write building name */
		if ( temp[0] + buildings_scroll == ANTIMATTER_CONDENSER ) {
			/* Special case; "Antimatter Condenser" is too long */
			gfx_PrintStringXY( "Antimatter", 215, temp[0] * 25 + 60 );
			gfx_PrintStringXY( "Condenser", 215, temp[0] * 25 + 68 );
		} else {
			/* Any normal building name */
			gfx_PrintStringXY( building_names[temp[0] + buildings_scroll], 215, temp[0] * 25 + 64 );
		}
	}
}

/* Update graphics that are updated constantly */
void update_graphics(void) {
	/* Text settings */
	gfx_SetTextFGColor( WHITE );
	gfx_SetTextBGColor( TRANSPARENT );
	gfx_SetTextTransparentColor( TRANSPARENT );
	
	/* Options for buying/selling */
	if ( buying ) {
		gfx_SetColor( BUTTON_GRAY );
		gfx_FillRectangle_NoClip( 301, 41, 17, 9 );
	} else {
		gfx_Sprite_NoClip( all, 301, 41 );
	}
	
	/* Print amount of cookies */
	gfx_GetSprite( cookies_back, 1, 52 );
	gfx_SetTextXY( 1, 52 );
	gfx_PrintInt( cookies, 7 );
	
	/* Draw cursor */
	gfx_GetSprite( cursor_back, cursor_x, cursor_y );
	gfx_TransparentSprite( cursor, cursor_x, cursor_y );
	gfx_BlitBuffer();
	
	/* Re-draw background of new graphics */
	gfx_Sprite( cursor_back, cursor_x, cursor_y );
	gfx_Sprite_NoClip( cookies_back, 1, 52 );
}

/* For now, check if a new second has gone by */
bool tick_timer(void) {
	if (timer_IntStatus & TIMER1_RELOADED) {
		timer_IntAcknowledge = TIMER1_RELOADED;
		return true;
	}
	return false;
}